-- @file Symbols.lua
-- @author darki
-- @copyright ↄ copyleft
-- @release 2018-10-15
--

-- new environ:
local myEnv = {}
-- inherit global:
setmetatable(myEnv, {__index = _ENV})

-- name defaults to 'Symbols' (while loading via dofile)
if _REQUIREDNAME == nil then
  Symbols = myEnv
else
  _ENV[_REQUIREDNAME] = myEnv
end

------------------------------------
-- using locals for speed optimization
------------------------------------
local insert  = table.insert
local remove  = table.remove
local math    = math

-- no more external/global access after this point
_ENV = myEnv

function indices (howmany)
    indarray = {}
    for i = 1, howmany do
        indarray[i] = i
    end
    return indarray
end

function shuffle(tbl) -- https://gist.github.com/Uradamus/10323382
  size = #tbl
  for i = size, 1, -1 do
    local rand = math.random(size)
    tbl[i], tbl[rand] = tbl[rand], tbl[i]
  end
  return tbl
end
local logfile=io.open("amc-compiled.log","r") if logfile ~= nil then io.close(logfile) print ("not setting random seed") else  math.randomseed( os.time() ) print("random seed set") end

print("sourcing Symbols")

function dump(o)    -- debug
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. dump(v) .. ',\n'
      end
      return s .. '} '
   else
      return tostring(o)
   end
end

function SymbolsFn ()
    switches_only = {}
    all_symbols   = {}
    do_draw = {}
    do_write = {}
    all_symbols = indices (71)    -- 2018-10-15 there are 71 symbols
    -- first three are switches: normally open, normally closed, two-way 
    for i=1,3 do
        insert ( switches_only, remove ( all_symbols, 1 ) )
    end

    all_symbols = shuffle ( all_symbols )
    switches_only = shuffle ( switches_only )

    for i=1,math.floor(#all_symbols /2)  do
        nextdraw = math.random(1,#all_symbols) 
        insert ( do_draw, remove ( all_symbols, nextdraw ) )
        -- debug:
        -- print ("nextdraw = " .. nextdraw .. " of " ..  #all_symbols .. "\n")

        nextwrite = math.random(1,#all_symbols) 
        insert ( do_write , remove ( all_symbols, nextwrite ) )
        -- debug:
        -- print ("nextwrite = " .. nextwrite .. " of " .. #all_symbols  .. "\n\n")
    end

    -------------- switches: to be inserted in EVERY test  -------
    at_position = math.random (1,15)
    remove ( do_draw, at_position )
    insert ( do_draw, at_position, remove ( switches_only ) )
    at_position = math.random (1,15)
    remove ( do_write, at_position )
    insert ( do_write, at_position, remove ( switches_only ) )
    --------------------------------------------------------------

    --[[
    -- debug, gotes to AMC log file if uncommented.
    c[slovo.A]=math.random(2,5) -- micro
	print ("do_draw:\n" )
	print (dump (do_draw))
	print ("do_write:\n" )
	print (dump (do_write))
    --]]

end

-- source only if _not_ ivnoked by AMC compilation:
local logfile=io.open("amc-compiled.log","r") if logfile ~= nil then io.close(logfile) else Symbols.SymbolsFn () end
return myEnv
